@@ -31,10 +31,7 @@ __all__ = [
 
 def _round_up_to_odd_integer(value):
     i = math.ceil(value)
-    if i % 2 == 0:
-        return i + 1
-    else:
-        return i
+    return i + 1 if i % 2 == 0 else i
 
 
 class Gaussian1DKernel(Kernel1D):
