@@ -8,7 +8,6 @@ core.py:
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
 
-
 import copy
 import csv
 import fnmatch
@@ -23,7 +22,7 @@ from collections import OrderedDict
 from contextlib import suppress
 from io import StringIO
 
-import numpy
+import numpy as np
 
 from astropy.table import Table
 from astropy.utils.data import get_readable_fileobj
@@ -158,7 +157,7 @@ class CsvWriter:
         return row_string
 
 
-class MaskedConstant(numpy.ma.core.MaskedConstant):
+class MaskedConstant(np.ma.core.MaskedConstant):
     """A trivial extension of numpy.ma.masked.
 
     We want to be able to put the generic term ``masked`` into a dictionary.
@@ -554,7 +553,7 @@ def _get_line_index(line_or_func, lines):
     """Return the appropriate line index, depending on ``line_or_func`` which
     can be either a function, a positive or negative int, or None.
     """
-    if hasattr(line_or_func, "__call__"):
+    if callable(line_or_func):
         return line_or_func(lines)
     elif line_or_func:
         if line_or_func >= 0:
@@ -763,15 +762,14 @@ class BaseHeader:
             and self.__class__.__name__ != "EcsvHeader"
         ):
             raise ValueError(
-                "Table format guessing requires at least two columns, got {}".format(
-                    list(self.colnames)
-                )
+                "Table format guessing requires at least two columns, "
+                f"got {list(self.colnames)}"
             )
 
         if names is not None and len(names) != len(self.colnames):
             raise InconsistentTableError(
-                "Length of names argument ({}) does not match number"
-                " of table columns ({})".format(len(names), len(self.colnames))
+                f"Length of names argument ({len(names)}) does not match number "
+                f"of table columns ({len(self.colnames)})"
             )
 
 
@@ -823,7 +821,7 @@ class BaseData:
             re_comment = re.compile(self.comment)
             return [x for x in nonblank_lines if not re_comment.match(x)]
         else:
-            return [x for x in nonblank_lines]
+            return list(nonblank_lines)
 
     def get_data_lines(self, lines):
         """
@@ -908,7 +906,7 @@ class BaseData:
         """READ: Replace string values in col.str_vals and set masks."""
         if self.fill_values:
             for col in (col for col in cols if col.fill_values):
-                col.mask = numpy.zeros(len(col.str_vals), dtype=bool)
+                col.mask = np.zeros(len(col.str_vals), dtype=bool)
                 for i, str_val in (
                     (i, x) for i, x in enumerate(col.str_vals) if x in col.fill_values
                 ):
@@ -955,7 +953,7 @@ class BaseData:
         lines : list
             List for collecting output of writing self.cols.
         """
-        if hasattr(self.start_line, "__call__"):
+        if callable(self.start_line):
             raise TypeError("Start_line attribute cannot be callable for write()")
         else:
             data_start_line = self.start_line or 0
@@ -1002,7 +1000,7 @@ def convert_numpy(numpy_type):
         the required type.
     """
     # Infer converter type from an instance of numpy_type.
-    type_name = numpy.array([], dtype=numpy_type).dtype.name
+    type_name = np.array([], dtype=numpy_type).dtype.name
     if "int" in type_name:
         converter_type = IntType
     elif "float" in type_name:
@@ -1020,26 +1018,26 @@ def convert_numpy(numpy_type):
         for any other string values.
         """
         if len(vals) == 0:
-            return numpy.array([], dtype=bool)
+            return np.array([], dtype=bool)
 
         # Try a smaller subset first for a long array
         if len(vals) > 10000:
-            svals = numpy.asarray(vals[:1000])
-            if not numpy.all(
+            svals = np.asarray(vals[:1000])
+            if not np.all(
                 (svals == "False") | (svals == "True") | (svals == "0") | (svals == "1")
             ):
                 raise ValueError('bool input strings must be False, True, 0, 1, or ""')
-        vals = numpy.asarray(vals)
+        vals = np.asarray(vals)
 
         trues = (vals == "True") | (vals == "1")
         falses = (vals == "False") | (vals == "0")
-        if not numpy.all(trues | falses):
+        if not np.all(trues | falses):
             raise ValueError('bool input strings must be only False, True, 0, 1, or ""')
 
         return trues
 
     def generic_converter(vals):
-        return numpy.array(vals, numpy_type)
+        return np.array(vals, numpy_type)
 
     converter = bool_converter if converter_type is BoolType else generic_converter
 
@@ -1068,7 +1066,7 @@ class BaseOutputter:
         try:
             # Don't allow list-like things that dtype accepts
             assert type(converters) is type
-            converters = [numpy.dtype(converters)]
+            converters = [np.dtype(converters)]
         except (AssertionError, TypeError):
             pass
 
@@ -1121,7 +1119,10 @@ class BaseOutputter:
 
                 converter_func, converter_type = col.converters[0]
                 if not issubclass(converter_type, col.type):
-                    raise TypeError("converter type does not match column type")
+                    raise TypeError(
+                        f"converter type {converter_type.__name__} does not match"
+                        f" column type {col.type.__name__} for column {col.name}"
+                    )
 
                 try:
                     col.data = converter_func(col.str_vals)
@@ -1175,7 +1176,14 @@ class TableOutputter(BaseOutputter):
     Output the table as an astropy.table.Table object.
     """
 
-    default_converters = [convert_numpy(int), convert_numpy(float), convert_numpy(str)]
+    default_converters = [
+        # Use `np.int64` to ensure large integers can be read as ints
+        # on platforms such as Windows
+        # https://github.com/astropy/astropy/issues/5744
+        convert_numpy(np.int64),
+        convert_numpy(float),
+        convert_numpy(str),
+    ]
 
     def __call__(self, cols, meta):
         # Sets col.data to numpy array and col.type to io.ascii Type class (e.g.
@@ -1183,8 +1191,8 @@ class TableOutputter(BaseOutputter):
         self._convert_vals(cols)
 
         t_cols = [
-            numpy.ma.MaskedArray(x.data, mask=x.mask)
-            if hasattr(x, "mask") and numpy.any(x.mask)
+            np.ma.MaskedArray(x.data, mask=x.mask)
+            if hasattr(x, "mask") and np.any(x.mask)
             else x.data
             for x in cols
         ]
@@ -1440,12 +1448,10 @@ class BaseReader(metaclass=MetaBaseReader):
                 # otherwise, we raise an error only if it is still inconsistent
                 if len(str_vals) != n_cols:
                     errmsg = (
-                        "Number of header columns ({}) inconsistent with"
-                        " data columns ({}) at data line {}\n"
-                        "Header values: {}\n"
-                        "Data values: {}".format(
-                            n_cols, len(str_vals), i, [x.name for x in cols], str_vals
-                        )
+                        f"Number of header columns ({n_cols}) inconsistent with "
+                        f"data columns ({len(str_vals)}) at data line {i}\n"
+                        f"Header values: {[x.name for x in cols]}\n"
+                        f"Data values: {str_vals}"
                     )
 
                     raise InconsistentTableError(errmsg)
@@ -1639,9 +1645,6 @@ class WhitespaceSplitter(DefaultSplitter):
 
 
 extra_reader_pars = (
-    "Reader",
-    "Inputter",
-    "Outputter",
     "delimiter",
     "comment",
     "quotechar",
@@ -1650,8 +1653,8 @@ extra_reader_pars = (
     "data_end",
     "converters",
     "encoding",
-    "data_Splitter",
-    "header_Splitter",
+    "data_splitter_cls",
+    "header_splitter_cls",
     "names",
     "include_names",
     "exclude_names",
@@ -1662,17 +1665,17 @@ extra_reader_pars = (
 )
 
 
-def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
+def _get_reader(reader_cls, inputter_cls=None, outputter_cls=None, **kwargs):
     """Initialize a table reader allowing for common customizations.  See ui.get_reader()
     for param docs.  This routine is for internal (package) use only and is useful
     because it depends only on the "core" module.
     """
     from .fastbasic import FastBasic
 
-    if issubclass(Reader, FastBasic):  # Fast readers handle args separately
-        if Inputter is not None:
-            kwargs["Inputter"] = Inputter
-        return Reader(**kwargs)
+    if issubclass(reader_cls, FastBasic):  # Fast readers handle args separately
+        if inputter_cls is not None:
+            kwargs["inputter_cls"] = inputter_cls
+        return reader_cls(**kwargs)
 
     # If user explicitly passed a fast reader with enable='force'
     # (e.g. by passing non-default options), raise an error for slow readers
@@ -1681,20 +1684,20 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
             raise ParameterError(
                 "fast_reader required with "
                 "{}, but this is not a fast C reader: {}".format(
-                    kwargs["fast_reader"], Reader
+                    kwargs["fast_reader"], reader_cls
                 )
             )
         else:
             del kwargs["fast_reader"]  # Otherwise ignore fast_reader parameter
 
     reader_kwargs = {k: v for k, v in kwargs.items() if k not in extra_reader_pars}
-    reader = Reader(**reader_kwargs)
+    reader = reader_cls(**reader_kwargs)
 
-    if Inputter is not None:
-        reader.inputter = Inputter()
+    if inputter_cls is not None:
+        reader.inputter = inputter_cls()
 
-    if Outputter is not None:
-        reader.outputter = Outputter()
+    if outputter_cls is not None:
+        reader.outputter = outputter_cls()
 
     # Issue #855 suggested to set data_start to header_start + default_header_length
     # Thus, we need to retrieve this from the class definition before resetting these numbers.
@@ -1741,10 +1744,10 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
             raise ValueError("header_start cannot be modified for this Reader")
     if "converters" in kwargs:
         reader.outputter.converters = kwargs["converters"]
-    if "data_Splitter" in kwargs:
-        reader.data.splitter = kwargs["data_Splitter"]()
-    if "header_Splitter" in kwargs:
-        reader.header.splitter = kwargs["header_Splitter"]()
+    if "data_splitter_cls" in kwargs:
+        reader.data.splitter = kwargs["data_splitter_cls"]()
+    if "header_splitter_cls" in kwargs:
+        reader.header.splitter = kwargs["header_splitter_cls"]()
     if "names" in kwargs:
         reader.names = kwargs["names"]
         if None in reader.names:
@@ -1789,7 +1792,7 @@ extra_writer_pars = (
 )
 
 
-def _get_writer(Writer, fast_writer, **kwargs):
+def _get_writer(writer_cls, fast_writer, **kwargs):
     """Initialize a table writer allowing for common customizations. This
     routine is for internal (package) use only and is useful because it depends
     only on the "core" module.
@@ -1803,15 +1806,15 @@ def _get_writer(Writer, fast_writer, **kwargs):
     if "fill_values" in kwargs and kwargs["fill_values"] is None:
         del kwargs["fill_values"]
 
-    if issubclass(Writer, FastBasic):  # Fast writers handle args separately
-        return Writer(**kwargs)
-    elif fast_writer and f"fast_{Writer._format_name}" in FAST_CLASSES:
+    if issubclass(writer_cls, FastBasic):  # Fast writers handle args separately
+        return writer_cls(**kwargs)
+    elif fast_writer and f"fast_{writer_cls._format_name}" in FAST_CLASSES:
         # Switch to fast writer
         kwargs["fast_writer"] = fast_writer
-        return FAST_CLASSES[f"fast_{Writer._format_name}"](**kwargs)
+        return FAST_CLASSES[f"fast_{writer_cls._format_name}"](**kwargs)
 
     writer_kwargs = {k: v for k, v in kwargs.items() if k not in extra_writer_pars}
-    writer = Writer(**writer_kwargs)
+    writer = writer_cls(**writer_kwargs)
 
     if "delimiter" in kwargs:
         writer.header.splitter.delimiter = kwargs["delimiter"]
