@@ -1,5 +1,6 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
+import io
 import locale
 import pathlib
 import platform
@@ -22,12 +23,16 @@ from astropy.units import Unit
 # NOTE: Python can be built without bz2.
 from astropy.utils.compat.optional_deps import HAS_BZ2
 from astropy.utils.data import get_pkg_data_path
-from astropy.utils.exceptions import AstropyWarning
+from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyWarning
 
 # setup/teardown function to have the tests run in the correct directory
-from .common import setup_function  # noqa: F401
-from .common import teardown_function  # noqa: F401
-from .common import assert_almost_equal, assert_equal, assert_true
+from .common import (
+    assert_almost_equal,
+    assert_equal,
+    assert_true,
+    setup_function,  # noqa: F401
+    teardown_function,  # noqa: F401
+)
 
 
 def asciiIO(x):
@@ -91,7 +96,7 @@ def test_read_specify_converters_with_names():
 
 def test_read_remove_and_rename_columns():
     csv_text = ["a,b,c", "1,2,3", "4,5,6"]
-    reader = ascii.get_reader(Reader=ascii.Csv)
+    reader = ascii.get_reader(reader_cls=ascii.Csv)
     reader.read(csv_text)
     header = reader.header
     with pytest.raises(KeyError, match="Column NOT-EXIST does not exist"):
@@ -134,8 +139,8 @@ def test_guess_with_names_arg():
 
 def test_guess_with_format_arg():
     """
-    When the format or Reader is explicitly given then disable the
-    strict column name checking in guessing.
+    When the format is explicitly given then disable the strict column name checking in
+    guessing.
     """
     dat = ascii.read(["1,2", "3,4"], format="basic")
     assert len(dat) == 1
@@ -145,14 +150,6 @@ def test_guess_with_format_arg():
     assert len(dat) == 1
     assert dat.colnames == ["a", "b"]
 
-    dat = ascii.read(["1,2", "3,4"], Reader=ascii.Basic)
-    assert len(dat) == 1
-    assert dat.colnames == ["1", "2"]
-
-    dat = ascii.read(["1,2", "3,4"], names=("a", "b"), Reader=ascii.Basic)
-    assert len(dat) == 1
-    assert dat.colnames == ["a", "b"]
-
     # For good measure check the same in the unified I/O interface
     dat = Table.read(["1,2", "3,4"], format="ascii.basic")
     assert len(dat) == 1
@@ -218,10 +215,10 @@ def test_read_all_files(fast_reader, path_format, home_is_data):
             if "guess" not in test_opts:
                 test_opts["guess"] = guess
             if (
-                "Reader" in test_opts
-                and f"fast_{test_opts['Reader']._format_name}" in core.FAST_CLASSES
+                "format" in test_opts
+                and f"fast_{test_opts['format']}" in core.FAST_CLASSES
             ):  # has fast version
-                if "Inputter" not in test_opts:  # fast reader doesn't allow this
+                if "inputter_cls" not in test_opts:  # fast reader doesn't allow this
                     test_opts["fast_reader"] = fast_reader
             table = ascii.read(testfile["name"], **test_opts)
             assert_equal(table.dtype.names, testfile["cols"])
@@ -246,9 +243,9 @@ def test_read_all_files_via_table(fast_reader, path_format, home_is_data):
             test_opts = testfile["opts"].copy()
             if "guess" not in test_opts:
                 test_opts["guess"] = guess
-            if "Reader" in test_opts:
-                format = f"ascii.{test_opts['Reader']._format_name}"
-                del test_opts["Reader"]
+            if "format" in test_opts:
+                format = f"ascii.{test_opts['format']}"
+                del test_opts["format"]
             else:
                 format = "ascii"
             if f"fast_{format}" in core.FAST_CLASSES:
@@ -267,7 +264,7 @@ def test_guess_all_files():
         if not testfile["opts"].get("guess", True):
             continue
         print(f"\n\n******** READING {testfile['name']}")
-        for filter_read_opts in (["Reader", "delimiter", "quotechar"], []):
+        for filter_read_opts in (["format", "delimiter", "quotechar"], []):
             # Copy read options except for those in filter_read_opts
             guess_opts = {
                 k: v for k, v in testfile["opts"].items() if k not in filter_read_opts
@@ -302,7 +299,7 @@ def test_validate_read_kwargs():
 
 def test_daophot_indef():
     """Test that INDEF is correctly interpreted as a missing value"""
-    table = ascii.read("data/daophot2.dat", Reader=ascii.Daophot)
+    table = ascii.read("data/daophot2.dat", format="daophot")
     for col in table.itercols():
         # Four columns have all INDEF values and are masked, rest are normal Column
         if col.name in ("OTIME", "MAG", "MERR", "XAIRMASS"):
@@ -317,17 +314,17 @@ def test_daophot_types():
     inferred automatically based only data values.  DAOphot reader uses
     the header information to assign types.
     """
-    table = ascii.read("data/daophot2.dat", Reader=ascii.Daophot)
+    table = ascii.read("data/daophot2.dat", format="daophot")
     assert table["LID"].dtype.char in "fd"  # float or double
     assert table["MAG"].dtype.char in "fd"  # even without any data values
     assert (
         table["PIER"].dtype.char in "US"
     )  # string (data values are consistent with int)
-    assert table["ID"].dtype.char in "il"  # int or long
+    assert table["ID"].dtype.kind == "i"  # int types: int, long, int64
 
 
 def test_daophot_header_keywords():
-    table = ascii.read("data/daophot.dat", Reader=ascii.Daophot)
+    table = ascii.read("data/daophot.dat", format="daophot")
     expected_keywords = (
         ("NSTARFILE", "test.nst.1", "filename", "%-23s"),
         ("REJFILE", '"hello world"', "filename", "%-23s"),
@@ -343,7 +340,7 @@ def test_daophot_header_keywords():
 
 
 def test_daophot_multiple_aperture():
-    table = ascii.read("data/daophot3.dat", Reader=ascii.Daophot)
+    table = ascii.read("data/daophot3.dat", format="daophot")
     assert "MAG5" in table.colnames  # MAG5 is one of the newly created column names
     assert table["MAG5"][4] == 22.13  # A sample entry in daophot3.dat file
     assert table["MERR2"][0] == 1.171
@@ -353,7 +350,7 @@ def test_daophot_multiple_aperture():
 
 
 def test_daophot_multiple_aperture2():
-    table = ascii.read("data/daophot4.dat", Reader=ascii.Daophot)
+    table = ascii.read("data/daophot4.dat", format="daophot")
     assert "MAG15" in table.colnames  # MAG15 is one of the newly created column name
     assert table["MAG15"][1] == -7.573  # A sample entry in daophot4.dat file
     assert table["MERR2"][0] == 0.049
@@ -365,7 +362,7 @@ def test_empty_table_no_header(fast_reader):
     with pytest.raises(ascii.InconsistentTableError):
         ascii.read(
             "data/no_data_without_header.dat",
-            Reader=ascii.NoHeader,
+            format="no_header",
             guess=False,
             fast_reader=fast_reader,
         )
@@ -533,7 +530,7 @@ def test_from_lines(fast_reader):
 
 
 def test_comment_lines():
-    table = ascii.get_reader(Reader=ascii.Rdb)
+    table = ascii.get_reader(reader_cls=ascii.Rdb)
     data = table.read("data/apostrophe.rdb")
     assert_equal(table.comment_lines, ["# first comment", "  # second comment"])
     assert_equal(data.meta["comments"], ["first comment", "second comment"])
@@ -673,7 +670,7 @@ def test_read_rdb_wrong_type(fast_reader):
 N\tN
 1\tHello"""
     with pytest.raises(ValueError):
-        ascii.read(table, Reader=ascii.Rdb, fast_reader=fast_reader)
+        ascii.read(table, format="rdb", fast_reader=fast_reader)
 
 
 @pytest.mark.parametrize("fast_reader", [True, False, "force"])
@@ -681,7 +678,7 @@ def test_default_missing(fast_reader):
     """
     Read a table with empty values and ensure that corresponding entries are masked
     """
-    table = "\n".join(
+    table = "\n".join(  # noqa: FLY002
         [
             "a,b,c,d",
             "1,3,,",
@@ -704,7 +701,7 @@ def test_default_missing(fast_reader):
     assert dat["a"].dtype.kind == "i"
 
     # Same test with a fixed width reader
-    table = "\n".join(
+    table = "\n".join(  # noqa: FLY002
         [
             " a   b   c   d ",
             "--- --- --- ---",
@@ -712,7 +709,7 @@ def test_default_missing(fast_reader):
             "  2     4.0  ss",
         ]
     )
-    dat = ascii.read(table, Reader=ascii.FixedWidthTwoLine)
+    dat = ascii.read(table, format="fixed_width_two_line")
     assert dat.masked is False
     assert dat.pformat() == [
         " a   b   c   d ",
@@ -721,7 +718,7 @@ def test_default_missing(fast_reader):
         "  2  -- 4.0  ss",
     ]
 
-    dat = ascii.read(table, Reader=ascii.FixedWidthTwoLine, fill_values=None)
+    dat = ascii.read(table, format="fixed_width_two_line", fill_values=None)
     assert dat.masked is False
     assert dat.pformat() == [
         " a   b   c   d ",
@@ -730,7 +727,7 @@ def test_default_missing(fast_reader):
         "  2     4.0  ss",
     ]
 
-    dat = ascii.read(table, Reader=ascii.FixedWidthTwoLine, fill_values=[])
+    dat = ascii.read(table, format="fixed_width_two_line", fill_values=[])
     assert dat.masked is False
     assert dat.pformat() == [
         " a   b   c   d ",
@@ -749,13 +746,13 @@ def get_testfiles(name=None):
             "cols": ("agasc_id", "n_noids", "n_obs"),
             "name": "data/apostrophe.rdb",
             "nrows": 2,
-            "opts": {"Reader": ascii.Rdb},
+            "opts": {"format": "rdb"},
         },
         {
             "cols": ("agasc_id", "n_noids", "n_obs"),
             "name": "data/apostrophe.tab",
             "nrows": 2,
-            "opts": {"Reader": ascii.Tab},
+            "opts": {"format": "tab"},
         },
         {
             "cols": (
@@ -774,7 +771,7 @@ def get_testfiles(name=None):
             ),
             "name": "data/cds.dat",
             "nrows": 1,
-            "opts": {"Reader": ascii.Cds},
+            "opts": {"format": "cds"},
         },
         {
             "cols": (
@@ -793,7 +790,7 @@ def get_testfiles(name=None):
             ),
             "name": "data/cds.dat",
             "nrows": 1,
-            "opts": {"Reader": ascii.Mrt},
+            "opts": {"format": "mrt"},
         },
         # Test malformed CDS file (issues #2241 #467)
         {
@@ -813,27 +810,27 @@ def get_testfiles(name=None):
             ),
             "name": "data/cds_malformed.dat",
             "nrows": 1,
-            "opts": {"Reader": ascii.Cds, "data_start": "guess"},
+            "opts": {"format": "cds", "data_start": "guess"},
         },
         {
             "cols": ("a", "b", "c"),
             "name": "data/commented_header.dat",
             "nrows": 2,
-            "opts": {"Reader": ascii.CommentedHeader},
+            "opts": {"format": "commented_header"},
         },
         {
             "cols": ("a", "b", "c"),
             "name": "data/commented_header2.dat",
             "nrows": 2,
-            "opts": {"Reader": ascii.CommentedHeader, "header_start": -1},
+            "opts": {"format": "commented_header", "header_start": -1},
         },
         {
             "cols": ("col1", "col2", "col3", "col4", "col5"),
             "name": "data/continuation.dat",
             "nrows": 2,
             "opts": {
-                "Inputter": ascii.ContinuationLinesInputter,
-                "Reader": ascii.NoHeader,
+                "inputter_cls": ascii.ContinuationLinesInputter,
+                "format": "no_header",
             },
         },
         {
@@ -852,7 +849,7 @@ def get_testfiles(name=None):
             ),
             "name": "data/daophot.dat",
             "nrows": 2,
-            "opts": {"Reader": ascii.Daophot},
+            "opts": {"format": "daophot"},
         },
         {
             "cols": (
@@ -865,13 +862,13 @@ def get_testfiles(name=None):
             ),
             "name": "data/sextractor.dat",
             "nrows": 3,
-            "opts": {"Reader": ascii.SExtractor},
+            "opts": {"format": "sextractor"},
         },
         {
             "cols": ("ra", "dec", "sai", "v2", "sptype"),
             "name": "data/ipac.dat",
             "nrows": 2,
-            "opts": {"Reader": ascii.Ipac},
+            "opts": {"format": "ipac"},
         },
         {
             "cols": (
@@ -916,7 +913,7 @@ def get_testfiles(name=None):
             ),
             "name": "data/no_data_cds.dat",
             "nrows": 0,
-            "opts": {"Reader": ascii.Cds},
+            "opts": {"format": "cds"},
         },
         {
             "cols": (
@@ -935,7 +932,7 @@ def get_testfiles(name=None):
             ),
             "name": "data/no_data_cds.dat",
             "nrows": 0,
-            "opts": {"Reader": ascii.Mrt},
+            "opts": {"format": "mrt"},
         },
         {
             "cols": (
@@ -953,25 +950,25 @@ def get_testfiles(name=None):
             ),
             "name": "data/no_data_daophot.dat",
             "nrows": 0,
-            "opts": {"Reader": ascii.Daophot},
+            "opts": {"format": "daophot"},
         },
         {
             "cols": ("NUMBER", "FLUX_ISO", "FLUXERR_ISO", "VALUES", "VALUES_1", "FLAG"),
             "name": "data/no_data_sextractor.dat",
             "nrows": 0,
-            "opts": {"Reader": ascii.SExtractor},
+            "opts": {"format": "sextractor"},
         },
         {
             "cols": ("ra", "dec", "sai", "v2", "sptype"),
             "name": "data/no_data_ipac.dat",
             "nrows": 0,
-            "opts": {"Reader": ascii.Ipac},
+            "opts": {"format": "ipac"},
         },
         {
             "cols": ("ra", "v2"),
             "name": "data/ipac.dat",
             "nrows": 2,
-            "opts": {"Reader": ascii.Ipac, "include_names": ["ra", "v2"]},
+            "opts": {"format": "ipac", "include_names": ["ra", "v2"]},
         },
         {
             "cols": ("a", "b", "c"),
@@ -983,13 +980,13 @@ def get_testfiles(name=None):
             "cols": ("agasc_id", "n_noids", "n_obs"),
             "name": "data/short.rdb",
             "nrows": 7,
-            "opts": {"Reader": ascii.Rdb},
+            "opts": {"format": "rdb"},
         },
         {
             "cols": ("agasc_id", "n_noids", "n_obs"),
             "name": "data/short.tab",
             "nrows": 7,
-            "opts": {"Reader": ascii.Tab},
+            "opts": {"format": "tab"},
         },
         {
             "cols": ("test 1a", "test2", "test3", "test4"),
@@ -1031,19 +1028,19 @@ def get_testfiles(name=None):
             "cols": ("col1", "col2", "col3", "col4", "col5", "col6"),
             "name": "data/simple4.txt",
             "nrows": 3,
-            "opts": {"Reader": ascii.NoHeader, "delimiter": "|"},
+            "opts": {"format": "no_header", "delimiter": "|"},
         },
         {
             "cols": ("col1", "col2", "col3"),
             "name": "data/space_delim_no_header.dat",
             "nrows": 2,
-            "opts": {"Reader": ascii.NoHeader},
+            "opts": {"format": "no_header"},
         },
         {
             "cols": ("col1", "col2", "col3"),
             "name": "data/space_delim_no_header.dat",
             "nrows": 2,
-            "opts": {"Reader": ascii.NoHeader, "header_start": None},
+            "opts": {"format": "no_header", "header_start": None},
         },
         {
             "cols": ("obsid", "offset", "x", "y", "name", "oaa"),
@@ -1073,38 +1070,38 @@ def get_testfiles(name=None):
             "name": "data/simple_csv.csv",
             "cols": ("a", "b", "c"),
             "nrows": 2,
-            "opts": {"Reader": ascii.Csv},
+            "opts": {"format": "csv"},
         },
         {
             "name": "data/simple_csv_missing.csv",
             "cols": ("a", "b", "c"),
             "nrows": 2,
             "skip": True,
-            "opts": {"Reader": ascii.Csv},
+            "opts": {"format": "csv"},
         },
         {
             "cols": ("cola", "colb", "colc"),
             "name": "data/latex1.tex",
             "nrows": 2,
-            "opts": {"Reader": ascii.Latex},
+            "opts": {"format": "latex"},
         },
         {
             "cols": ("Facility", "Id", "exposure", "date"),
             "name": "data/latex2.tex",
             "nrows": 3,
-            "opts": {"Reader": ascii.AASTex},
+            "opts": {"format": "aastex"},
         },
         {
             "cols": ("cola", "colb", "colc"),
             "name": "data/latex3.tex",
             "nrows": 2,
-            "opts": {"Reader": ascii.Latex},
+            "opts": {"format": "latex"},
         },
         {
             "cols": ("Col1", "Col2", "Col3", "Col4"),
             "name": "data/fixed_width_2_line.txt",
             "nrows": 2,
-            "opts": {"Reader": ascii.FixedWidthTwoLine},
+            "opts": {"format": "fixed_width_two_line"},
         },
     ]
 
@@ -1116,7 +1113,7 @@ def get_testfiles(name=None):
                 "cols": ("Column 1", "Column 2", "Column 3"),
                 "name": "data/html.html",
                 "nrows": 3,
-                "opts": {"Reader": ascii.HTML},
+                "opts": {"format": "html"},
             }
         )
     except ImportError:
@@ -1179,7 +1176,7 @@ def test_sextractor_units():
     """
     Make sure that the SExtractor reader correctly inputs descriptions and units.
     """
-    table = ascii.read("data/sextractor2.dat", Reader=ascii.SExtractor, guess=False)
+    table = ascii.read("data/sextractor2.dat", format="sextractor", guess=False)
     expected_units = [
         None,
         Unit("pix"),
@@ -1211,7 +1208,7 @@ def test_sextractor_last_column_array():
     """
     Make sure that the SExtractor reader handles the last column correctly when it is array-like.
     """
-    table = ascii.read("data/sextractor3.dat", Reader=ascii.SExtractor, guess=False)
+    table = ascii.read("data/sextractor3.dat", format="sextractor", guess=False)
     expected_columns = [
         "X_IMAGE",
         "Y_IMAGE",
@@ -1352,7 +1349,13 @@ def test_guessing_file_object():
 def test_pformat_roundtrip():
     """Check that the screen output of ``print tab`` can be read. See #3025."""
     """Read a table with empty values and ensure that corresponding entries are masked"""
-    table = "\n".join(["a,b,c,d", "1,3,1.11,1", "2, 2, 4.0 , ss "])
+    table = "\n".join(  # noqa: FLY002
+        [
+            "a,b,c,d",
+            "1,3,1.11,1",
+            "2, 2, 4.0 , ss ",
+        ]
+    )
     dat = ascii.read(table)
     out = ascii.read(dat.pformat())
     assert len(dat) == len(out)
@@ -1535,8 +1538,8 @@ def test_data_header_start(fast_reader):
             assert t.colnames == ["a", "b"]
             assert len(t) == 1
             assert np.all(t["a"] == [1])
-            # Sanity check that the expected Reader is being used
-            assert get_read_trace()[-1]["kwargs"]["Reader"] is (
+            # Sanity check that the expected reader_cls is being used
+            assert get_read_trace()[-1]["kwargs"]["reader_cls"] is (
                 ascii.Basic if (fast_reader is False) else ascii.FastBasic
             )
 
@@ -1563,9 +1566,9 @@ def test_table_with_no_newline():
 
     # Put a single line of column names but with no newline
     for kwargs in [
-        dict(),
-        dict(guess=False, fast_reader=False, format="basic"),
-        dict(guess=False, fast_reader=True, format="fast_basic"),
+        {},
+        {"guess": False, "fast_reader": False, "format": "basic"},
+        {"guess": False, "fast_reader": True, "format": "fast_basic"},
     ]:
         table = BytesIO()
         table.write(b"a b")
@@ -1579,7 +1582,7 @@ def test_path_object():
     data = ascii.read(fpath)
 
     assert len(data) == 2
-    assert sorted(list(data.columns)) == ["test 1a", "test2", "test3", "test4"]
+    assert sorted(data.columns) == ["test 1a", "test2", "test3", "test4"]
     assert data["test2"][1] == "hat2"
 
 
@@ -1651,7 +1654,7 @@ def test_initial_column_fill_values():
     class Tester(ascii.Basic):
         header_class = TestHeader
 
-    reader = ascii.get_reader(Reader=Tester)
+    reader = ascii.get_reader(reader_cls=Tester)
 
     assert (
         reader.read(
@@ -1848,8 +1851,8 @@ def test_read_non_ascii():
 def test_kwargs_dict_guess(enable):
     """Test that fast_reader dictionary is preserved through guessing sequence."""
     # Fails for enable=(True, 'force') - #5578
-    ascii.read("a\tb\n 1\t2\n3\t 4.0", fast_reader=dict(enable=enable))
-    assert get_read_trace()[-1]["kwargs"]["Reader"] is (
+    ascii.read("a\tb\n 1\t2\n3\t 4.0", fast_reader={"enable": enable})
+    assert get_read_trace()[-1]["kwargs"]["reader_cls"] is (
         ascii.Tab if (enable is False) else ascii.FastTab
     )
     for k in get_read_trace():
@@ -1921,8 +1924,8 @@ def test_include_names_rdb_fast():
     lines[0] = "a\ta_2\ta_1\ta_3\ta_4"
     dat = ascii.read(lines, fast_reader="force", include_names=["a", "a_2", "a_3"])
     assert len(dat) == 2
-    assert dat["a"].dtype == int
-    assert dat["a_2"].dtype == int
+    assert dat["a"].dtype.kind == "i"
+    assert dat["a_2"].dtype.kind == "i"
 
 
 @pytest.mark.parametrize("fast_reader", [False, "force"])
@@ -2064,3 +2067,48 @@ def test_read_converters_simplified():
             t2 = Table.read(
                 out.getvalue(), format="ascii.basic", converters=converters, guess=False
             )
+
+
+def test_read_deprecations():
+    def check_warns(func, *args):
+        with pytest.warns(AstropyDeprecationWarning) as warns:
+            out = func(
+                *args,
+                Reader=ascii.Basic,
+                Inputter=ascii.BaseInputter,
+                Outputter=ascii.TableOutputter,
+                header_Splitter=ascii.DefaultSplitter,
+                data_Splitter=ascii.DefaultSplitter,
+            )
+            assert len(warns) == 5
+            for kwarg in (
+                "Reader",
+                "Inputter",
+                "Outputter",
+                "header_Splitter",
+                "data_Splitter",
+            ):
+                msg = f'"{kwarg}" was deprecated'
+                assert any(warn.message.args[0].startswith(msg) for warn in warns)
+            return out
+
+    tbl = check_warns(ascii.read, ["a b", "1 2"])
+    assert tbl.pformat_all() == [" a   b ", "--- ---", "  1   2"]
+
+    reader = check_warns(ascii.get_reader)
+    tbl = reader.read(["a b", "1 2"])
+    assert tbl.pformat_all() == [" a   b ", "--- ---", "  1   2"]
+
+
+def test_write_deprecations():
+    t = simple_table()
+    out = io.StringIO()
+    with pytest.warns(AstropyDeprecationWarning, match='"Writer" was deprecated'):
+        ascii.write(t, out, Writer=ascii.Csv)
+    assert out.getvalue().splitlines() == ["a,b,c", "1,1.0,c", "2,2.0,d", "3,3.0,e"]
+
+    with pytest.warns(AstropyDeprecationWarning, match='"Writer" was deprecated'):
+        writer = ascii.get_writer(Writer=ascii.Csv)
+    out = io.StringIO()
+    writer.write(t, out)
+    assert out.getvalue().splitlines() == ["a,b,c", "1,1.0,c", "2,2.0,d", "3,3.0,e"]
