@@ -360,9 +360,10 @@ class TestDiff(FitsTestCase):
         hdu = fits.CompImageHDU(data=data)
         hdu.writeto(self.temp("test.fits"))
 
-        with fits.open(self.temp("test.fits")) as hdula, fits.open(
-            self.temp("test.fits")
-        ) as hdulb:
+        with (
+            fits.open(self.temp("test.fits")) as hdula,
+            fits.open(self.temp("test.fits")) as hdulb,
+        ):
             diff = FITSDiff(hdula, hdulb)
             assert diff.identical
 
@@ -381,7 +382,7 @@ class TestDiff(FitsTestCase):
         assert "Data dimensions differ" in report
         assert "a: 10 x 10" in report
         assert "b: 100" in report
-        assert "No further data comparison performed."
+        assert "No further data comparison performed." in report
 
     def test_different_pixels(self):
         ia = np.arange(100).reshape(10, 10)
@@ -532,7 +533,7 @@ class TestDiff(FitsTestCase):
         assert "Table rows differ" in report
         assert "a: 2" in report
         assert "b: 3" in report
-        assert "No further data comparison performed."
+        assert "No further data comparison performed." in report
 
     def test_different_table_data(self):
         """
