@@ -10,42 +10,54 @@ Pontzen, who has granted the Astropy project permission to use the
 code under a BSD license.
 """
 
-# Import order matters here -- circular dependencies abound!
-# Lots of things to import - go from more basic to advanced, so that
-# whatever advanced ones need generally has been imported already;
-# this also makes it easier to understand where most time is spent
-# (e.g., using python -X importtime).
-
-from .core import set_enabled_units
-
-# isort: off
-from .core import *
-from .quantity import *
-
-from . import astrophys, cgs, misc, photometric, si
-from .function import units as function_units
-
-from .si import *
+from . import (
+    astrophys,
+    cgs,
+    core,
+    decorators,
+    misc,
+    photometric,
+    physical,
+    quantity,
+    si,
+    structured,
+)
 from .astrophys import *
-from .photometric import *
 from .cgs import *
-from .physical import *
-from .function.units import *
-from .misc import *
-
-from .equivalencies import *
-
-from .function.core import *
-from .function.logarithmic import *
-
+from .core import *
+from .core import set_enabled_units
 from .decorators import *
+from .misc import *
+from .photometric import *
+from .physical import *
+from .quantity import *
+from .si import *
 from .structured import *
 
-# isort: on
+# Import order matters here -- circular dependencies abound!
+# isort: split
+from . import equivalencies, function
+from .equivalencies import *
+from .function import *
 
-del bases
+__all__ = []
+__all__ += core.__all__
+__all__ += quantity.__all__
+__all__ += decorators.__all__
+__all__ += structured.__all__
+__all__ += si.__all__
+__all__ += cgs.__all__
+__all__ += astrophys.__all__
+__all__ += physical.__all__
+__all__ += misc.__all__
+__all__ += photometric.__all__
+__all__ += structured.__all__
+__all__ += equivalencies.__all__
+__all__ += function.__all__
+# We sort `__all__` for the docs. We use `globals` to avoid confusing any static
+# analysis tools.
+globals()["__all__"].sort()
 
 # Enable the set of default units.  This notably does *not* include
 # Imperial units.
-
-set_enabled_units([si, cgs, astrophys, function_units, misc, photometric])
+set_enabled_units([si, cgs, astrophys, function.units, misc, photometric])
