@@ -3,7 +3,9 @@
 
 Functions, including ufuncs, are tested in test_functions.py
 """
+
 import operator
+import sys
 
 import numpy as np
 import pytest
@@ -12,7 +14,7 @@ from numpy.testing import assert_array_equal
 from astropy import units as u
 from astropy.coordinates import Longitude
 from astropy.units import Quantity
-from astropy.utils.compat import NUMPY_LT_1_22
+from astropy.utils.compat.optional_deps import HAS_PLT
 from astropy.utils.masked import Masked, MaskedNDArray
 
 
@@ -289,40 +291,59 @@ class TestMaskedNDArraySubclassCreation:
         assert_array_equal(ma.unmasked, self.a.view(np.ndarray))
         assert_array_equal(ma.mask, self.m)
 
+    def test_viewing_independent_shape(self):
+        mms = Masked(self.a, mask=self.m)
+        mms2 = mms.view()
+        mms2.shape = mms2.shape[::-1]
+        assert mms2.shape == mms.shape[::-1]
+        assert mms2.mask.shape == mms.shape[::-1]
+        # This should not affect the original array!
+        assert mms.shape == self.a.shape
+        assert mms.mask.shape == self.a.shape
+
 
 class TestMaskedQuantityInitialization(TestMaskedArrayInitialization, QuantitySetup):
+    @classmethod
+    def setup_class(self):
+        super().setup_class()
+        # Ensure we have used MaskedQuantity before - just in case a single test gets
+        # called; see gh-15316.
+        self.MQ = Masked(Quantity)
+
+    def test_masked_quantity_getting(self):
+        # First check setup_class (or previous use) defined a cache entry.
+        mcls = Masked._masked_classes[type(self.a)]
+        # Next check this is what one gets now.
+        MQ = Masked(Quantity)
+        assert MQ is mcls
+        assert issubclass(MQ, Quantity)
+        assert issubclass(MQ, Masked)
+        # And also what one gets implicitly.
+        mq = Masked([1.0, 2.0] * u.m)
+        assert isinstance(mq, MQ)
+
     def test_masked_quantity_class_init(self):
-        # TODO: class definitions should be more easily accessible.
-        mcls = Masked._masked_classes[self.a.__class__]
         # This is not a very careful test.
-        mq = mcls([1.0, 2.0], mask=[True, False], unit=u.s)
+        mq = self.MQ([1.0, 2.0], mask=[True, False], unit=u.s)
         assert mq.unit == u.s
         assert np.all(mq.value.unmasked == [1.0, 2.0])
         assert np.all(mq.value.mask == [True, False])
         assert np.all(mq.mask == [True, False])
 
-    def test_masked_quantity_getting(self):
-        mcls = Masked._masked_classes[self.a.__class__]
-        MQ = Masked(Quantity)
-        assert MQ is mcls
-
     def test_initialization_without_mask(self):
         # Default for not giving a mask should be False.
-        MQ = Masked(Quantity)
-        mq = MQ([1.0, 2.0], u.s)
+        mq = self.MQ([1.0, 2.0], u.s)
         assert mq.unit == u.s
         assert np.all(mq.value.unmasked == [1.0, 2.0])
         assert np.all(mq.mask == [False, False])
 
     @pytest.mark.parametrize("masked_array", [Masked, np.ma.MaskedArray])
     def test_initialization_with_masked_values(self, masked_array):
-        MQ = Masked(Quantity)
         a = np.array([1.0, 2.0])
         m = np.array([True, False])
         ma = masked_array(a, m)
-        mq = MQ(ma)
-        assert isinstance(mq, Masked)
-        assert isinstance(mq, Quantity)
+        mq = self.MQ(ma)
+        assert isinstance(mq, self.MQ)
         assert_array_equal(mq.value.unmasked, a)
         assert_array_equal(mq.mask, m)
 
@@ -425,13 +446,22 @@ class TestViewing(MaskedArraySetup):
         assert_array_equal(ma2.unmasked, self.a.view("c8"))
         assert_array_equal(ma2.mask, self.mask_a)
 
-    @pytest.mark.parametrize("new_dtype", ["2f4", "f8,f8,f8"])
+    def test_viewing_as_new_structured_dtype(self):
+        ma2 = self.ma.view("f8,f8,f8")
+        assert_array_equal(ma2.unmasked, self.a.view("f8,f8,f8"))
+        assert_array_equal(ma2.mask, self.mask_a.view("?,?,?"))
+        # Check round-trip
+        ma3 = ma2.view(self.ma.dtype)
+        assert_array_equal(ma3.unmasked, self.ma.unmasked)
+        assert_array_equal(ma3.mask, self.mask_a)
+
+    @pytest.mark.parametrize("new_dtype", ["f4", "2f4"])
     def test_viewing_as_new_dtype_not_implemented(self, new_dtype):
         # But cannot (yet) view in way that would need to create a new mask,
         # even though that view is possible for a regular array.
         check = self.a.view(new_dtype)
         with pytest.raises(NotImplementedError, match="different.*size"):
-            self.ma.view(check.dtype)
+            self.ma.view(new_dtype)
 
     def test_viewing_as_something_impossible(self):
         with pytest.raises(TypeError):
@@ -632,6 +662,15 @@ class MaskedItemTests(MaskedArraySetup):
         assert_array_equal(base.unmasked, self.a)
         assert_array_equal(base.mask, expected_mask)
 
+    @pytest.mark.parametrize("item", VARIOUS_ITEMS)
+    def test_hash(self, item):
+        ma_part = self.ma[item]
+        if ma_part.ndim > 0 or ma_part.mask.any():
+            with pytest.raises(TypeError, match="unhashable"):
+                hash(ma_part)
+        else:
+            assert hash(ma_part) == hash(ma_part.unmasked)
+
 
 class TestMaskedArrayItems(MaskedItemTests):
     @classmethod
@@ -688,7 +727,7 @@ class MaskedOperatorTests(MaskedArraySetup):
 
     def test_not_implemented(self):
         with pytest.raises(TypeError):
-            self.ma > "abc"
+            self.ma > "abc"  # noqa: B015
 
     @pytest.mark.parametrize("different_names", [False, True])
     @pytest.mark.parametrize("op", (operator.eq, operator.ne))
@@ -733,6 +772,31 @@ class MaskedOperatorTests(MaskedArraySetup):
         expected_mask3 = np.logical_or.outer(np.zeros(3, bool), mask1)
         assert_array_equal(result3.mask, expected_mask3)
 
+    def test_matmul_axes(self):
+        m1 = Masked(np.arange(27.0).reshape(3, 3, 3))
+        m2 = Masked(np.arange(-27.0, 0.0).reshape(3, 3, 3))
+        mxm1 = np.matmul(m1, m2)
+        exp = np.matmul(m1.unmasked, m2.unmasked)
+        assert_array_equal(mxm1.unmasked, exp)
+        assert_array_equal(mxm1.mask, False)
+        m1.mask[0, 1, 2] = True
+        m2.mask[0, 2, 0] = True
+        axes = [(0, 2), (-2, -1), (0, 1)]
+        mxm2 = np.matmul(m1, m2, axes=axes)
+        exp2 = np.matmul(m1.unmasked, m2.unmasked, axes=axes)
+        # Any unmasked result will have all elements contributing unity,
+        # while masked entries mean the total will be lower.
+        mask2 = (
+            np.matmul(
+                (~m1.mask).astype(int),
+                (~m2.mask).astype(int),
+                axes=axes,
+            )
+            != m1.shape[axes[0][1]]
+        )
+        assert_array_equal(mxm2.unmasked, exp2)
+        assert_array_equal(mxm2.mask, mask2)
+
     def test_matvec(self):
         result = self.ma @ self.mb
         assert np.all(result.mask)
@@ -785,7 +849,7 @@ class TestMaskedArrayOperators(MaskedOperatorTests):
 
     def test_not_implemented(self):
         with pytest.raises(TypeError):
-            Masked(["a", "b"]) > object()
+            Masked(["a", "b"]) > object()  # noqa: B015
 
 
 class TestMaskedQuantityOperators(MaskedOperatorTests, QuantitySetup):
@@ -828,6 +892,16 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         assert_array_equal(ma_sum.unmasked, expected_data)
         assert_array_equal(ma_sum.mask, expected_mask)
 
+    def test_sum_hash(self):
+        ma_sum = self.ma.sum()
+        assert ma_sum.mask
+        # Masked scalars cannot be hashed.
+        with pytest.raises(TypeError, match="unhashable"):
+            hash(ma_sum)
+        ma_sum2 = Masked(self.a).sum()
+        # But an unmasked scalar can.
+        assert hash(ma_sum2) == hash(self.a.sum())
+
     @pytest.mark.parametrize("axis", (0, 1, None))
     def test_cumsum(self, axis):
         ma_sum = self.ma.cumsum(axis)
@@ -898,6 +972,10 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         assert_array_equal(ma_mean.unmasked, expected_data)
         assert_array_equal(ma_mean.mask, expected_mask)
 
+    def test_mean_hash(self):
+        ma_mean = self.ma.mean()
+        assert hash(ma_mean) == hash(ma_mean.unmasked[()])
+
     @pytest.mark.filterwarnings("ignore:.*encountered in.*divide")
     @pytest.mark.parametrize("axis", (0, 1, None))
     def test_var(self, axis):
@@ -1048,11 +1126,9 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         ma = Masked(data=[1, 2], mask=[True, False])
         assert ma.argmin() == 1
 
-    if not NUMPY_LT_1_22:
-
-        def test_argmin_keepdims(self):
-            ma = Masked(data=[[1, 2], [3, 4]], mask=[[True, False], [False, False]])
-            assert_array_equal(ma.argmin(axis=0, keepdims=True), np.array([[1, 0]]))
+    def test_argmin_keepdims(self):
+        ma = Masked(data=[[1, 2], [3, 4]], mask=[[True, False], [False, False]])
+        assert_array_equal(ma.argmin(axis=0, keepdims=True), np.array([[1, 0]]))
 
     @pytest.mark.parametrize("axis", (0, 1, None))
     def test_argmax(self, axis):
@@ -1062,11 +1138,9 @@ class TestMaskedArrayMethods(MaskedArraySetup):
         expected_data = filled.argmax(axis)
         assert_array_equal(ma_argmax, expected_data)
 
-    if not NUMPY_LT_1_22:
-
-        def test_argmax_keepdims(self):
-            ma = Masked(data=[[1, 2], [3, 4]], mask=[[True, False], [False, False]])
-            assert_array_equal(ma.argmax(axis=1, keepdims=True), np.array([[1], [1]]))
+    def test_argmax_keepdims(self):
+        ma = Masked(data=[[1, 2], [3, 4]], mask=[[True, False], [False, False]])
+        assert_array_equal(ma.argmax(axis=1, keepdims=True), np.array([[1], [1]]))
 
     @pytest.mark.parametrize("axis", (0, 1, None))
     def test_argsort(self, axis):
@@ -1290,17 +1364,46 @@ class TestMaskedArrayProductMethods(MaskedArraySetup):
         assert_array_equal(ma_sum.mask, expected_mask)
 
 
-def test_masked_str_explicit():
+def test_masked_str_repr_explicit_float():
+    sa = np.array([np.pi, 2 * np.pi])
+    msa = Masked(sa, [False, True])
+    # Test masking  the array works as expected, including truncating digits
+    assert str(msa) == "[3.14159265        ———]"
+    # Test the digits are kept for scalars.
+    assert str(msa[0]) == "3.141592653589793" == str(sa[0])
+    # And that the masked string has the same length.
+    assert str(msa[1]) == "              ———"
+    # Test temporary precision change (which does not affect scalars).
+    with np.printoptions(precision=3, floatmode="fixed"):
+        assert str(msa) == "[3.142   ———]"
+        assert str(msa[0]) == "3.141592653589793" == str(sa[0])
+    assert repr(msa) == "MaskedNDArray([3.14159265,        ———])"
+
+
+def test_masked_str_explicit_string():
+    sa = np.array(["2001-02-03", "2002-03-04"])
+    msa = Masked(sa, [False, True])
+    assert str(msa) == "['2001-02-03'          ———]"
+    assert str(msa[0]) == "2001-02-03" == str(sa[0])
+    assert str(msa[1]) == "       ———"
+    byteorder = "<" if sys.byteorder == "little" else ">"
+    repr_ = f"MaskedNDArray(['2001-02-03',          ———], dtype='{byteorder}U10')"
+    assert repr(msa) == repr_
+
+
+def test_masked_str_explicit_structured():
     sa = np.array([(1.0, 2.0), (3.0, 4.0)], dtype="f8,f8")
     msa = Masked(sa, [(False, True), (False, False)])
     assert str(msa) == "[(1., ——) (3., 4.)]"
     assert str(msa[0]) == "(1., ——)"
-    assert str(msa[1]) == "(3., 4.)"
+    assert str(msa[1]) == "(3., 4.)" == str(sa[1])
     with np.printoptions(precision=3, floatmode="fixed"):
         assert str(msa) == "[(1.000,   ———) (3.000, 4.000)]"
+        assert str(msa[0]) == "(1.000,   ———)"
+        assert str(msa[1]) == "(3.000, 4.000)" == str(sa[1])
 
 
-def test_masked_repr_explicit():
+def test_masked_repr_explicit_structured():
     # Use explicit endianness to ensure tests pass on all architectures
     sa = np.array([(1.0, 2.0), (3.0, 4.0)], dtype=">f8,>f8")
     msa = Masked(sa, [(False, True), (False, False)])
@@ -1424,3 +1527,33 @@ class TestMaskedQuantityInteractionWithNumpyMA(
     TestMaskedArrayInteractionWithNumpyMA, QuantitySetup
 ):
     pass
+
+
+@pytest.mark.skipif(not HAS_PLT, reason="requires matplotlib.pyplot")
+def test_plt_scatter_masked():
+    import matplotlib as mpl
+
+    from astropy.utils import minversion
+
+    MPL_LT_3_8 = not minversion(mpl, "3.8")
+
+    if MPL_LT_3_8:
+        pytest.skip("never worked before matplotlib 3.8")
+
+    # check that plotting Masked data doesn't raise an exception
+    # see https://github.com/astropy/astropy/issues/12481
+    import matplotlib.pyplot as plt
+
+    _, ax = plt.subplots()
+
+    # no mask
+    x = Masked([1, 2, 3])
+    ax.scatter(x, x, c=x)
+
+    # all masked
+    x = Masked([1, 2, 3], mask=True)
+    ax.scatter(x, x, c=x)
+
+    # *some* masked
+    x = Masked([1, 2, 3], mask=[False, True, False])
+    ax.scatter(x, x, c=x)
