@@ -19,15 +19,17 @@ import urllib.parse
 import urllib.request
 import warnings
 from concurrent.futures import ThreadPoolExecutor
+from contextlib import nullcontext
 from itertools import islice
 from tempfile import NamedTemporaryFile, TemporaryDirectory
 
-import py.path
 import pytest
 
 import astropy.utils.data
 from astropy import units as _u  # u is taken
 from astropy.config import paths
+from astropy.tests.helper import CI, IS_CRON, PYTEST_LT_8_0
+from astropy.utils.compat.optional_deps import HAS_BZ2, HAS_LZMA
 from astropy.utils.data import (
     CacheDamaged,
     CacheMissingWarning,
@@ -59,15 +61,11 @@ from astropy.utils.data import (
 )
 from astropy.utils.exceptions import AstropyWarning
 
-CI = os.environ.get("CI", "false") == "true"
 TESTURL = "http://www.astropy.org"
 TESTURL2 = "http://www.astropy.org/about.html"
 TESTURL_SSL = "https://www.astropy.org"
 TESTLOCAL = get_pkg_data_filename(os.path.join("data", "local.dat"))
 
-# NOTE: Python can be built without bz2 or lzma.
-from astropy.utils.compat.optional_deps import HAS_BZ2, HAS_LZMA
-
 # For when we need "some" test URLs
 FEW = 5
 
@@ -108,7 +106,7 @@ def valid_urls(tmp_path):
     def _valid_urls(tmp_path):
         for i in itertools.count():
             c = os.urandom(16).hex()
-            fn = tmp_path / f"valid_{str(i)}"
+            fn = tmp_path / f"valid_{i}"
             with open(fn, "w") as f:
                 f.write(c)
             u = url_to(fn)
@@ -121,7 +119,7 @@ def valid_urls(tmp_path):
 def invalid_urls(tmp_path):
     def _invalid_urls(tmp_path):
         for i in itertools.count():
-            fn = tmp_path / f"invalid_{str(i)}"
+            fn = tmp_path / f"invalid_{i}"
             if not os.path.exists(fn):
                 yield url_to(fn)
 
@@ -233,7 +231,7 @@ def test_download_file_absolute_path(valid_urls, temp_cache):
     assert is_abs(download_file(u, cache=False))  # no cache
     assert is_abs(download_file(u, cache=True))  # not in cache
     assert is_abs(download_file(u, cache=True))  # in cache
-    for k, v in cache_contents().items():
+    for v in cache_contents().values():
         assert is_abs(v)
 
 
@@ -350,10 +348,8 @@ def test_download_with_sources_and_bogus_original(
     # it was loaded by mistake.
     for i, (um, c_bad) in enumerate(islice(valid_urls, FEW)):
         assert not is_url_in_cache(um)
-        sources[um] = []
         # For many of them the sources list starts with invalid URLs
-        for iu in islice(invalid_urls, i):
-            sources[um].append(iu)
+        sources[um] = list(islice(invalid_urls, i))
         u, c = next(valid_urls)
         sources[um].append(u)
         urls.append((um, c, c_bad))
@@ -365,7 +361,7 @@ def test_download_with_sources_and_bogus_original(
         )
     else:
         rs = [
-            download_file(u, cache=True, sources=sources.get(u, None))
+            download_file(u, cache=True, sources=sources.get(u))
             for (u, c, c_bad) in urls
         ]
     assert len(rs) == len(urls)
@@ -390,8 +386,8 @@ def test_download_file_threaded_many(temp_cache, valid_urls):
         r = list(P.map(lambda u: download_file(u, cache=True), [u for (u, c) in urls]))
     check_download_cache()
     assert len(r) == len(urls)
-    for r, (u, c) in zip(r, urls):
-        assert get_file_contents(r) == c
+    for r_, (u, c) in zip(r, urls):
+        assert get_file_contents(r_) == c
 
 
 @pytest.mark.skipif(
@@ -442,11 +438,11 @@ def test_download_file_threaded_many_partial_success(
         r = list(P.map(get, urls))
     check_download_cache()
     assert len(r) == len(urls)
-    for r, u in zip(r, urls):
+    for r_, u in zip(r, urls):
         if u in contents:
-            assert get_file_contents(r) == contents[u]
+            assert get_file_contents(r_) == contents[u]
         else:
-            assert r is None
+            assert r_ is None
 
 
 def test_clear_download_cache(valid_urls):
@@ -736,9 +732,7 @@ def test_download_parallel_with_sources_and_bogus_original(
     sources = {}
     for i, (um, c_bad) in enumerate(islice(valid_urls, FEW)):
         assert not is_url_in_cache(um)
-        sources[um] = []
-        for iu in islice(invalid_urls, i):
-            sources[um].append(iu)
+        sources[um] = list(islice(invalid_urls, i))
         u, c = next(valid_urls)
         sources[um].append(u)
         urls.append((um, c, c_bad))
@@ -756,8 +750,8 @@ def test_download_parallel_many(temp_cache, valid_urls):
 
     r = download_files_in_parallel([u for (u, c) in td])
     assert len(r) == len(td)
-    for r, (u, c) in zip(r, td):
-        assert get_file_contents(r) == c
+    for r_, (_, c) in zip(r, td):
+        assert get_file_contents(r_) == c
 
 
 def test_download_parallel_partial_success(temp_cache, valid_urls, invalid_urls):
@@ -907,7 +901,7 @@ def test_find_by_hash(valid_urls, temp_cache):
 def test_find_invalid():
     # this is of course not a real data file and not on any remote server, but
     # it should *try* to go to the remote server
-    with pytest.raises(urllib.error.URLError):
+    with pytest.raises((urllib.error.URLError, TimeoutError)):
         get_pkg_data_filename(
             "kjfrhgjklahgiulrhgiuraehgiurhgiuhreglhurieghruelighiuerahiulruli"
         )
@@ -1104,10 +1098,14 @@ def test_data_noastropy_fallback(monkeypatch):
     assert os.path.isfile(fnout)
 
     # clearing the cache should be a no-up that doesn't affect fnout
-    with pytest.warns(
-        CacheMissingWarning, match=r".*Not clearing data cache - cache inaccessible.*"
-    ):
+    with pytest.warns(CacheMissingWarning) as record:
         clear_download_cache(TESTURL)
+    assert len(record) == 2
+    assert (
+        record[0].message.args[0]
+        == "Remote data cache could not be accessed due to OSError"
+    )
+    assert "Not clearing data cache - cache inaccessible" in record[1].message.args[0]
     assert os.path.isfile(fnout)
 
     # now remove it so tests don't clutter up the temp dir this should get
@@ -1148,12 +1146,11 @@ def test_read_unicode(filename):
     contents = get_pkg_data_contents(os.path.join("data", filename), encoding="binary")
     assert isinstance(contents, bytes)
     x = contents.splitlines()[1]
-    # fmt: off
-    assert x == (
+    expected = (
         b"\xff\xd7\x94\xd7\x90\xd7\xa1\xd7\x98\xd7\xa8\xd7\x95\xd7\xa0\xd7\x95"
         b"\xd7\x9e\xd7\x99 \xd7\xa4\xd7\x99\xd7\x99\xd7\xaa\xd7\x95\xd7\x9f"[1:]
     )
-    # fmt: on
+    assert x == expected
 
 
 def test_compressed_stream():
@@ -1605,12 +1602,6 @@ def test_get_fileobj_str(a_file):
         assert rf.read() == c
 
 
-def test_get_fileobj_localpath(a_file):
-    fn, c = a_file
-    with get_readable_fileobj(py.path.local(fn)) as rf:
-        assert rf.read() == c
-
-
 def test_get_fileobj_pathlib(a_file):
     fn, c = a_file
     with get_readable_fileobj(pathlib.Path(fn)) as rf:
@@ -1779,7 +1770,7 @@ def test_can_make_directories_readonly(tmp_path):
             )
         elif platform.system() == "Windows":
             pytest.skip(
-                "It seems we can't make a driectory un-writable under Windows "
+                "It seems we can't make a directory un-writable under Windows "
                 "with chmod, in spite of the documentation."
             )
         else:
@@ -2077,9 +2068,14 @@ def test_removal_of_open_files_windows(temp_cache, valid_urls, monkeypatch):
         # This platform is able to remove files while in use.
         monkeypatch.setattr(astropy.utils.data, "_rmtree", no_rmtree)
 
+    if PYTEST_LT_8_0:
+        ctx = nullcontext()
+    else:
+        ctx = pytest.warns(CacheMissingWarning, match=".*PermissionError.*")
+
     u, c = next(valid_urls)
     with open(download_file(u, cache=True)):
-        with pytest.warns(CacheMissingWarning, match=r".*in use.*"):
+        with pytest.warns(CacheMissingWarning, match=".*in use.*"), ctx:
             clear_download_cache(u)
 
 
@@ -2092,10 +2088,15 @@ def test_update_of_open_files_windows(temp_cache, valid_urls, monkeypatch):
         # This platform is able to remove files while in use.
         monkeypatch.setattr(astropy.utils.data, "_rmtree", no_rmtree)
 
+    if PYTEST_LT_8_0:
+        ctx = nullcontext()
+    else:
+        ctx = pytest.warns(CacheMissingWarning, match=".*read-only.*")
+
     u, c = next(valid_urls)
     with open(download_file(u, cache=True)):
         u2, c2 = next(valid_urls)
-        with pytest.warns(CacheMissingWarning, match=r".*in use.*"):
+        with pytest.warns(CacheMissingWarning, match=".*in use.*"), ctx:
             f = download_file(u, cache="update", sources=[u2])
         check_download_cache()
         assert is_url_in_cache(u)
@@ -2190,7 +2191,7 @@ def test_clear_download_cache_raises_os_error(temp_cache, valid_urls, monkeypatc
 
 
 @pytest.mark.skipif(
-    CI and os.environ.get("IS_CRON", "false") == "false",
+    CI and not IS_CRON,
     reason="Flaky/too much external traffic for regular CI",
 )
 @pytest.mark.remote_data
