@@ -186,11 +186,9 @@ class CoordinateHelper:
 
         self.grid_lines_kwargs.update(kwargs)
 
-        if self.grid_lines_kwargs["visible"]:
-            if not draw_grid:
-                self.grid_lines_kwargs["visible"] = False
-        else:
-            self.grid_lines_kwargs["visible"] = True
+        if draw_grid is None:
+            draw_grid = True
+        self.grid_lines_kwargs["visible"] = draw_grid
 
     def set_coord_type(self, coord_type, coord_wrap=None):
         """
@@ -628,18 +626,23 @@ class CoordinateHelper:
 
         renderer.close_group("grid lines")
 
-    def _draw_ticks(self, renderer, bboxes, ticklabels_bbox):
+    def _draw_ticks(self, renderer, existing_bboxes):
         """
         Draw all ticks and ticklabels.
+
+        Parameters
+        ----------
+        existing_bboxes : list[Bbox]
+            All bboxes for ticks that have already been drawn by other
+            coordinates.
         """
         renderer.open_group("ticks")
         self.ticks.draw(renderer)
-        self.ticklabels.draw(
-            renderer,
-            bboxes=bboxes,
-            ticklabels_bbox=ticklabels_bbox,
-            tick_out_size=self.ticks.out_size,
-        )
+
+        self.ticklabels._tick_out_size = self.ticks.out_size
+
+        self.ticklabels._set_existing_bboxes(existing_bboxes)
+        self.ticklabels.draw(renderer)
 
         renderer.close_group("ticks")
 
